@@ -22,6 +22,10 @@ WHERE CONTEXT *Context;
 
 WHERE char Errorbuf[SHORT_STRING];
 
+#if defined(DL_STANDALONE) && defined(USE_DOTLOCK)
+WHERE char *MuttDotlock;
+#endif
+
 WHERE char *AliasFile;
 WHERE char *AliasFmt;
 WHERE char *AttachSep;
@@ -44,6 +48,7 @@ WHERE char *Hostname;
 WHERE char *ImapUser INITVAL (NULL);
 WHERE char *ImapPass INITVAL (NULL);
 WHERE short ImapCheckTime;
+WHERE char *ImapHomeNamespace INITVAL (NULL);
 #endif
 WHERE char *InReplyTo;
 WHERE char *Inbox;
