@@ -17,15 +17,19 @@
  */ 
 
 #ifndef _IMAP_H
-#define _IMAP_H
+#define _IMAP_H 1
 
-extern int imap_append_message (CONTEXT *ctx, MESSAGE *msg);
-extern int imap_check_mailbox (CONTEXT *ctx, int *index_hint);
-extern int imap_fetch_message (MESSAGE *msg, CONTEXT *ctx, int msgno);
-extern int imap_open_mailbox (CONTEXT *ctx);
-extern int imap_open_mailbox_append (CONTEXT *ctx);
-extern int imap_sync_mailbox (CONTEXT *ctx);
-extern void imap_fastclose_mailbox (CONTEXT *ctx);
-extern int imap_buffy_check (char *path);
+#include "browser.h"
+
+int imap_append_message (CONTEXT *ctx, MESSAGE *msg);
+int imap_check_mailbox (CONTEXT *ctx, int *index_hint);
+int imap_fetch_message (MESSAGE *msg, CONTEXT *ctx, int msgno);
+int imap_open_mailbox (CONTEXT *ctx);
+int imap_open_mailbox_append (CONTEXT *ctx);
+int imap_sync_mailbox (CONTEXT *ctx);
+void imap_fastclose_mailbox (CONTEXT *ctx);
+int imap_buffy_check (char *path);
+int imap_subscribe (char *path, int subscribe);
+int imap_init_browse (char *path, struct browser_state *state);
 
 #endif
