@@ -95,11 +95,15 @@ struct option_t MuttVars[] = {
   { "date_format",	DT_STR,	 R_BOTH, UL &DateFmt, UL "!%a, %b %d, %Y at %I:%M:%S%p %Z" },
   { "default_hook",	DT_STR,	 R_NONE, UL &DefaultHook, UL "~f %s !~P | (~P ~C %s)" },
   { "delete",		DT_QUAD, R_NONE, OPT_DELETE, M_ASKYES },
+#if defined(DL_STANDALONE) && defined(USE_DOTLOCK)
+  { "dotlock_program",  DT_PATH, R_NONE, UL &MuttDotlock, UL BINDIR "/mutt_dotlock" },
+#endif
   { "dsn_notify",	DT_STR,	 R_NONE, UL &DsnNotify, UL "" },
   { "dsn_return",	DT_STR,	 R_NONE, UL &DsnReturn, UL "" },
   { "edit_headers",	DT_BOOL, R_NONE, OPTEDITHDRS, 0 },
   { "edit_hdrs",	DT_SYN,  R_NONE, UL "edit_headers", 0 },
   { "editor",		DT_PATH, R_NONE, UL &Editor, 0 },
+  { "encode_from",	DT_BOOL, R_NONE, OPTENCODEFROM, 0 },
   { "escape",		DT_STR,	 R_NONE, UL &EscChar, UL "~" },
   { "fast_reply",	DT_BOOL, R_NONE, OPTFASTREPLY, 0 },
   { "fcc_attach",	DT_BOOL, R_NONE, OPTFCCATTACH, 1 },
@@ -121,6 +125,7 @@ struct option_t MuttVars[] = {
   { "help",		DT_BOOL, R_BOTH, OPTHELP, 1 },
   { "hidden_host",	DT_BOOL, R_NONE, OPTHIDDENHOST, 0 },
   { "history",		DT_NUM,	 R_NONE, UL &HistSize, 10 },
+  { "honor_followup_to", DT_QUAD, R_NONE, OPT_MFUPTO, M_YES },
   { "hostname",		DT_STR,	 R_NONE, UL &Fqdn, 0 },
 #ifdef USE_IMAP
   { "imap_checkinterval", 	DT_NUM,	 R_NONE, UL &ImapCheckTime, 0 },
@@ -128,6 +133,7 @@ struct option_t MuttVars[] = {
   { "imap_user",	DT_STR,  R_NONE, UL &ImapUser, UL 0 },
   { "imap_pass", 	DT_STR,  R_NONE, UL &ImapPass, UL 0 },
   { "imap_passive",		DT_BOOL, R_NONE, OPTIMAPPASSIVE, 1 },
+  { "imap_home_namespace",	DT_STR, R_NONE, UL &ImapHomeNamespace, UL 0},
 #endif
   { "implicit_autoview", DT_BOOL,R_NONE, OPTIMPLICITAUTOVIEW, 0},
   { "in_reply_to",	DT_STR,	 R_NONE, UL &InReplyTo, UL "%i; from %n on %{!%a, %b %d, %Y at %I:%M:%S%p %Z}" },
@@ -181,6 +187,8 @@ struct option_t MuttVars[] = {
   { "pgp_long_ids",	DT_BOOL, R_NONE, OPTPGPLONGIDS, 0 },
   { "pgp_replyencrypt",	DT_BOOL, R_NONE, OPTPGPREPLYENCRYPT, 0 },
   { "pgp_replysign",	DT_BOOL, R_NONE, OPTPGPREPLYSIGN, 0 },
+  { "pgp_replysignencrypted", DT_BOOL, R_NONE, OPTPGPREPLYSIGNENCRYPTED, 0 },
+  { "pgp_retainable_sigs", DT_BOOL, R_NONE, OPTPGPRETAINABLESIG, 0 },
   { "pgp_show_unusable", DT_BOOL, R_NONE, OPTPGPSHOWUNUSABLE, 1 },
   { "pgp_sign_as",	DT_STR,	 R_NONE, UL &PgpSignAs, 0 },
   { "pgp_sign_micalg",	DT_STR,	 R_NONE, UL &PgpSignMicalg, UL "pgp-md5" },
@@ -283,12 +291,13 @@ struct option_t MuttVars[] = {
   { "use_8bitmime",	DT_BOOL, R_NONE, OPTUSE8BITMIME, 0 },
   { "use_domain",	DT_BOOL, R_NONE, OPTUSEDOMAIN, 1 },
   { "use_from",		DT_BOOL, R_NONE, OPTUSEFROM, 1 },
+  { "user_agent",	DT_BOOL, R_NONE, OPTXMAILER, 1},
   { "visual",		DT_PATH, R_NONE, UL &Visual, 0 },
   { "wait_key",		DT_BOOL, R_NONE, OPTWAITKEY, 1 },
   { "wrap_search",	DT_BOOL, R_NONE, OPTWRAPSEARCH, 1 },
   { "write_inc",	DT_NUM,	 R_NONE, UL &WriteInc, 10 },
   { "write_bcc",	DT_BOOL, R_NONE, OPTWRITEBCC, 1},
-  { NULL }
+    { NULL }
 };
 
 const struct mapping_t SortMethods[] = {
