@@ -38,6 +38,8 @@
 
 #ifdef HAVE_SYSEXITS_H
 #include <sysexits.h>
+#else /* Make sure EX_OK is defined <philiph@pobox.com> */
+#define EX_OK 0
 #endif
 
 extern char RFC822Specials[];
@@ -584,10 +586,9 @@ static CONTENT *mutt_get_content_info (const char *fname, BODY *b)
       {
         if (linelen == 2 && ch != 'r') from = 0;
         else if (linelen == 3 && ch != 'o') from = 0;
-        else if (linelen == 4 && ch != 'm') from = 0;
-        else if (linelen == 5)
+        else if (linelen == 4)
 	{
-          if (ch == ' ') info->from = 1;
+          if (ch == 'm') info->from = 1;
           from = 0;
         }
       }
@@ -843,14 +844,14 @@ static void mutt_set_encoding (BODY *b, CONTENT *info)
 {
   if (b->type == TYPETEXT)
   {
-    if (info->lobin)
+    if (info->lobin || (info->from && option (OPTENCODEFROM)))
       b->encoding = ENCQUOTEDPRINTABLE;
     else if (info->hibin)
       b->encoding = option (OPTALLOW8BIT) ? ENC8BIT : ENCQUOTEDPRINTABLE;
     else
       b->encoding = ENC7BIT;
   }
-  else if (b->type == TYPEMESSAGE)
+  else if (b->type == TYPEMESSAGE || b->type == TYPEMULTIPART)
   {
     if (info->lobin || info->hibin)
     {
@@ -862,7 +863,8 @@ static void mutt_set_encoding (BODY *b, CONTENT *info)
     else
       b->encoding = ENC7BIT;
   }
-  else if (info->lobin || info->hibin || info->binary || info->linemax > 990)
+  else if (info->lobin || info->hibin || info->binary || info->linemax > 990
+	   || (option (OPTENCODEFROM) && info->from))
   {
     /* Determine which encoding is smaller  */
     if (1.33 * (float)(info->lobin+info->hibin+info->ascii) < 3.0 * (float) (info->lobin + info->hibin) + (float)info->ascii)
@@ -1336,13 +1338,11 @@ int mutt_write_rfc822_header (FILE *fp, ENVELOPE *env, BODY *attach,
     mutt_write_mime_header (attach, fp);
   }
 
-#ifndef NO_XMAILER
-  if (mode == 0 && !privacy)
+  if (mode == 0 && !privacy && option (OPTXMAILER))
   {
     /* Add a vanity header */
-    fprintf (fp, "X-Mailer: Mutt %s\n", MUTT_VERSION);
+    fprintf (fp, "User-Agent: Mutt/%s\n", MUTT_VERSION);
   }
-#endif
 
   /* Add any user defined headers */
   for (; tmp; tmp = tmp->next)
