@@ -1,4 +1,8 @@
-static const char rcsid[]="$Id: thread.c,v 2.0 1998/12/12 08:07:37 roessler Rel $";
+/*
+ * WORK IN PROGRESS ALERT: Daniel Eisenbud <daniel@math.berkeley.edu>
+ * is currently working on this code.  Contact him before working on it!
+ */
+
 /*
  * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  *
@@ -24,17 +28,26 @@ static const char rcsid[]="$Id: thread.c,v 2.0 1998/12/12 08:07:37 roessler Rel
 #include <ctype.h>
 
 /* This function makes use of the fact that Mutt stores message references in
-   reverse order (i.e., last to first).  This is optiminal since we would like
-   to find the most recent message to which "cur" refers itself.  */
+ * reverse order (i.e., last to first).  This is optiminal since we would like
+ * to find the most recent message to which "cur" refers itself.  
+ */
+
 static HEADER *find_reference (HEADER *cur, CONTEXT *ctx)
 {
   LIST *refs = cur->env->references;
-  void *ptr;
+  HEADER *ptr;
 
   for (; refs; refs = refs->next)
+  {
+    /* ups, this message is in a reference loop. bad. */
+
+    if (cur->env->message_id && !strcmp (cur->env->message_id, refs->data))
+      continue;
+
     if ((ptr = hash_find (ctx->id_hash, refs->data)))
       return ptr;
-
+  }
+  
   return NULL;
 }
 
@@ -43,18 +56,25 @@ static int need_display_subject (CONTEXT *ctx, HEADER *tree)
 {
   HEADER *tmp;
 
+  /* if our subject is different from our parent's, display it */
   if (tree->subject_changed)
     return (1);
+
+  /* if our subject is different from that of our closest previously displayed
+   * sibling, display the subject */
   for (tmp = tree->prev; tmp; tmp = tmp->prev)
   {
     if (tmp->virtual >= 0 || (tmp->collapsed && (!ctx->pattern || tmp->limited)))
     {
-      if (!tmp->subject_changed)
-	return (0);
+      if (tmp->subject_changed)
+	return (1);
       else
 	break;
     }
   }
+  
+  /* if there is a parent-to-child subject change anywhere between us and our
+   * closest displayed ancestor, display the subject */
   for (tmp = tree->parent; tmp; tmp = tmp->parent)
   {
     if (tmp->virtual >= 0 || (tmp->collapsed && (!ctx->pattern || tmp->limited)))
@@ -62,7 +82,9 @@ static int need_display_subject (CONTEXT *ctx, HEADER *tree)
     else if (tmp->subject_changed)
       return (1);
   }
-  return (0);
+  
+  /* if we have no visible parent or previous sibling, display the subject */
+  return (1);
 }
 
 /* determines whether a later sibling or the child of a later 
@@ -735,6 +757,7 @@ int _mutt_traverse_thread (CONTEXT *ctx, HEADER *cur, int flag)
 
   if (flag & (M_THREAD_COLLAPSE | M_THREAD_UNCOLLAPSE))
   {
+    cur->pair = 0; /* force index entry's color to be re-evaluated */
     cur->collapsed = flag & M_THREAD_COLLAPSE;
     if (cur->virtual != -1)
     {
@@ -761,6 +784,7 @@ int _mutt_traverse_thread (CONTEXT *ctx, HEADER *cur, int flag)
   {
     if (flag & (M_THREAD_COLLAPSE | M_THREAD_UNCOLLAPSE))
     {
+      cur->pair = 0; /* force index entry's color to be re-evaluated */
       cur->collapsed = flag & M_THREAD_COLLAPSE;
       if (!roothdr && CHECK_LIMIT)
       {
